/* stylelint-disable docusaurus/copyright-header */

/**
 * CSS files with the .module.css suffix will be treated as CSS modules
 * and scoped locally.
 */
 .heroBanner {
  padding: 4rem 0;
  text-align: center;
  position: relative;
  overflow: hidden;
  --ifm-hero-text-color: #fff;
}

@media screen and (max-width: 966px) {
  .heroBanner {
    padding: 2rem;
  }
}

.buttons {
  display: flex;
  align-items: center;
  justify-content: center;
}

.features {
  display: flex;
  align-items: center;
  padding: 2rem 0;
  width: 100%;
}

.featureImage {
  height: 200px;
  width: 200px;
}

.funcs {
  margin-bottom: 80px;
}

.funcSet {
  margin-bottom: 32px;
}

.funcSetHeading {
  margin-bottom: 8px;
}

.funcSetItems {
  columns: 2;
}
@media screen and (min-width: 500px) {
  .funcSetItems {
    columns: 3;
  }
}
@media screen and (min-width: 750px) {
  .funcSetItems {
    columns: 4;
  }
}
@media screen and (min-width: 966px) {
  .funcSetItems {
    columns: 5;
  }
}


.funcSetItem {
  display: block;
  margin-bottom: 8px;
  margin-right: 8px;
}