/* stylelint-disable docusaurus/copyright-header */
/* @import './preflight.css'; */
@import url("https://cdn.rawgit.com/tonsky/FiraCode/1.205/distr/fira_code.css");
@import url('https://fonts.googleapis.com/css2?family=Chivo');
@import url('https://fonts.googleapis.com/css2?family=Lato');

@import url('https://fonts.googleapis.com/css2?family=Fira+Mono');
@import url('https://fonts.googleapis.com/css2?family=Bungee&display=swap');

@-webkit-keyframes glow {
  from {
    text-shadow: 0 0 8px #427FD4, 0 0 10px #39F8FF;
  }

  to {
    text-shadow: 0 0 10px #427FD4, 0 0 11px #39F8FF;
  }
}

@keyframes glow {
  to {
    text-shadow: 0 0 10px #427FD4, 0 0 11px #39F8FF;
  }
}

html[data-theme='dark'] .navbar__title {
  font-family: "Bungee";
  letter-spacing: 0.1px;
  line-height: 1.3rem;
  color: #FDFDFD;
  font-weight: 500;
  font-size: 20px;
  animation: glow 3s ease-in-out infinite alternate;
}
html[data-theme='light'] .navbar__title {
  font-family: "Bungee";
  letter-spacing: 0.1px;
  line-height: 1.3rem;
  font-weight: 500;
  font-size: 20px;
  animation: glow 3s ease-in-out infinite alternate;
}


body {
  font-family: "Lato";
  /*color: #1f1f1f;*/
  font-size: 16px;
}
.menu__link{
  font-family: "Fira Mono";
  letter-spacing: 0.6px;
  line-height: 1.2rem; /* increases, so wrapping headers don't overlap */
  font-weight: 500;
  font-size: 14px;
}
.code{
  font-family: "Fira Mono";
  line-height: 1.8rem; /* increases, so wrapping headers don't overlap */
  font-weight: 500;
  font-size: 11px;
}
h1, h2, .h1, .h2 {
  font-family: "Chivo";
  letter-spacing: 0.6px;
  line-height: 2.2rem; /* increases, so wrapping headers don't overlap */
  font-weight: 500;
  font-size: 13px;
}

h3, h4, h5, h6, .h3, .h4{
  font-family: "Chivo";
  letter-spacing: 0.6px;
  line-height: 1.8rem; /* increases, so wrapping headers don't overlap */
  font-weight: 500;
  font-size: 14px;
}
.navbar__item.navbar__link,.navbar__link.navbar__link--active.navbar__link--active{
  font-family: "Fira Mono";
  letter-spacing: 0.6px;
  line-height: 1.8rem; /* increases, so wrapping headers don't overlap */
  font-weight: 500;
  font-size: 16px;
}

pre, code {
  font-family: "Fira Mono", Consolas, Inconsolata, monospace;
}

/**
 * Any CSS included here will be global. The classic template
 * bundles Infima by default. Infima is a CSS framework designed to
 * work well for content-centric websites.
 */


button,
input,
optgroup,
select,
textarea {
  font-family: inherit; /* 1 */
  font-size: 100%; /* 1 */
  line-height: 1.15; /* 1 */
  margin: 0; /* 2 */
}

button,
input,
optgroup,
select,
textarea {
  padding: 0;
  line-height: inherit;
  color: inherit;
}


/**
* Show the overflow in IE.
* 1. Show the overflow in Edge.
*/

button,
input { /* 1 */
  overflow: visible;
}


/**
* Remove the inheritance of text transform in Edge, Firefox, and IE.
* 1. Remove the inheritance of text transform in Firefox.
*/

button,
select { /* 1 */
  text-transform: none;
}


/**
* Correct the inability to style clickable types in iOS and Safari.
*/

button,
[type="button"],
[type="reset"],
[type="submit"] {
  -webkit-appearance: button;
}

/**
  * Remove the inner border and padding in Firefox.
  */

button::-moz-focus-inner,
[type="button"]::-moz-focus-inner,
[type="reset"]::-moz-focus-inner,
[type="submit"]::-moz-focus-inner {
  border-style: none;
  padding: 0;
}

/**
  * Restore the focus styles unset by the previous rule.
  */

button:-moz-focusring,
[type="button"]:-moz-focusring,
[type="reset"]:-moz-focusring,
[type="submit"]:-moz-focusring {
  outline: 1px dotted ButtonText;
}


button {
  background-color: transparent;
  background-image: none;
  padding: 0;
  border: none;
  font-weight: bold;
}

/**
* Work around a Firefox/IE bug where the transparent `button` background
* results in a loss of the default `button` focus styles.
*/

button:focus {
  outline: 1px dotted;
  outline: 5px auto -webkit-focus-ring-color;
}

button,
[role="button"] {
  cursor: pointer;
}

::before,
::after {
  box-sizing: border-box; /* 1 */
  border-width: 0; /* 2 */
  border-style: solid; /* 2 */
  border-color: #e2e8f0; /* 2 */
}
b {
  font-family: 'Bungee';
  animation: glow 3s ease-in-out infinite alternate;
}
a.menu__link.menu__link--active {
  animation: glow 3s ease-in-out infinite alternate;
}
a.table-of-contents__link.table-of-contents__link--active {
  animation: glow 3s ease-in-out infinite alternate;
}
/* You can override the default Infima variables here. */
:root {
  --ifm-color-primary: #014F84;
  --ifm-color-primary-dark:	#3F72AF;
  --ifm-color-primary-darker: hsl(271, 100%, 14%);
  --ifm-color-primary-darkest: hsl(262, 100%, 8%);
  --ifm-color-primary-light: hsl(7deg, 100%, 76%);
  --ifm-color-primary-lighter: hsl(7deg, 100%, 82%);
  --ifm-color-primary-lightest: hsl(7deg, 100%, 90%);
  /* --ifm-color-primary-dark: rgb(33, 175, 144);
  --ifm-color-primary-darker: rgb(31, 165, 136);
  --ifm-color-primary-darkest: rgb(26, 136, 112);
  --ifm-color-primary-light: rgb(70, 203, 174);
  --ifm-color-primary-lighter: rgb(102, 212, 189);
  --ifm-color-primary-lightest: rgb(146, 224, 208); */
  --ifm-code-font-size: 95%;

  --ifm-link-color: var(--ifm-color-primary-dark);
  --ifm-menu-color-active: var(--ifm-link-color);
  --ifm-navbar-link-hover-color: var(--ifm-link-color);
}

html[data-theme='dark'] {
  --ifm-color-primary: #014F84;
}

.docusaurus-highlight-code-line {
  background-color: hsl(228, 50%, 5%);

  display: block;
  margin: 0 calc(-1 * var(--ifm-pre-padding));

  padding: 0 var(--ifm-pre-padding);
}

.monaco-editor .contents {
  font-size: inherit;
  margin-bottom: inherit;
  padding-top: inherit;
  padding-bottom: inherit;
}

.code-table {
  --ifm-table-cell-padding: 2px;
  font-size: 0.875rem;
  display: table;
}
.text-sm { font-size: 0.875rem; }
.text-xs { font-size: 0.75rem; }
.limited-code-block > div {
  max-height: 35vh;
  overflow: auto;
  border-radius: 5px;
  font-size: var(--ifm-code-font-size);

}





/** baby tailwind */
:root {
  --border-color-default: #ddd;
}

.h-full { height: 100% }
.flex { display: flex; }
.flex-col { flex-direction: column; }
.flex-grow { flex-grow: 1;}
.flex-grow-0 { flex-grow: 0;}
.items-center { align-items: center; }
.justify-between { justify-content: space-between; }

.grid { display: grid; }
.grid-cols-12 { grid-template-columns: repeat(12, minmax(0, 1fr)); }
.gap-2 { gap: 0.5rem; }
.col-auto { grid-column: auto; }